﻿######
# . '.\SMVI_Mount_Post.ps1' -NAFilerIP 158.228.1.2 -NAVolume SMVI_test_Volume -SMVIJobName SMVIBackup01 -VMVCenter VirtualCenter01 -VMDKPath "smvit01/smvit01_1.vmdk" -BackupMountPoint "C:\BackupMounts\smvit01\" -Pause
######

<#
	.SYNOPSIS
	This script will unmount the temporary flexclone of the SMVI Backup VMDK 
	from the current VM, remove the Datastore from the ESX Hosts and destroy it.
	
	.DESCRIPTION
	The script will unmount the temporary FlexClone that was created of the most 
	recent SMVI backup job from the VM, remove the .vmdk file from the vm, remove 
	the datastore from the ESX Hosts of the current VM, then destroy the temporary
	FlexClone.
	
	.EXAMPLE
	PS C:\>.\SMVI_Mount_Post.ps1 -NAFilerIP 158.228.1.2 -NAVolume SMVI_test_Volume -SMVIJobName SMVIBackup01 -VMVCenter VirtualCenter01 -VMDKPath "smvit01/smvit01_1.vmdk" -BackupMountPoint "C:\BackupMounts\smvit01\" -Pause
	
	.NOTES
	Name      :  SMVI_Mount_Post.ps1
	Author    :  Jeff Burdzel
	E-Mail    :  Burdzy@GMail.com
	Last Edit :  June 29, 2012 3:07:23 PM
	• SMVIFunctions.ps1 must be in the current folder.
	• Please set 'Set-PowerCLIConfiguration -InvalidCertificateAction Ignore' for future compatibility.
	• Please add NetApp user credentials for account that will be running the Script
	     Add-NaCredential -Controller $NAFilerIP -Credential root
	• Please run script as an account with the appropriate access to Virtual Center to prevent being prompted. 
#>


param(
	[parameter(Mandatory=$true, HelpMessage="IP of NetApp Filer volume resides on.")]
		[string]
		#IP of NetApp Filer volume resides on.
		$NAFilerIP,
	[parameter(Mandatory=$true, HelpMessage="Name of volume on Filer.")]
		[string]
		#Name of volume on Filer.
		$NAVolume,
	[parameter(Mandatory=$true, HelpMessage="SMVI Job name.")]
		[string]
		#SMVI Job name.
		$SMVIJobName,
	[parameter(Mandatory=$true, HelpMessage="Name of Virtual Center the current VM resides on.")]
		[string]
		#Name of Virtual Center the current VM resides on.
		$VMVCenter,
	[parameter(Mandatory=$true, HelpMessage="Path to VMDK file within datastore.")]
		[string]
		#Path to VMDK file within datastore.
		$VMDKPath,
	[parameter(Mandatory=$true, HelpMessage="Local folder path VMDK was mounted to.")]
		[string]
		#Local folder path VMDK was mounted to.
		$BackupMountPoint,
	[parameter(Mandatory=$false, HelpMessage="Name of Flex Clone Volume.")]
		[string]
		#Name of Flex Clone Volume.
		$FlexCloneName = "$($NAVolume)_nw",
	[parameter(Mandatory=$false, HelpMessage="Pause after writing each status (Before action occurs).")]
		[Switch]
		#Pause after writing each status (Before action occurs).
		$Pause = $false,
	[parameter(Mandatory=$false, HelpMessage="Set if VMWare Consistent Snapshot is disabled.")]
		[Switch]
		#Set if VMWare Consistent Snapshot is disabled.
		$NoVMSnap = $false
)

#################################
# Load Functions                #
#################################
Write-Host "$(Get-Date -Format 'yyyy.MM.dd HH:mm:ss') Loading SMVIFunction.ps1"
if (!(Test-Path "SMVIFunctions.ps1")) {
	LogErrorExit "Please copy SMVIFunctions.ps1 to the current folder." 1
}

. .\SMVIFunctions.ps1

#################################
# Load PowerCLI & DataONTap     #
# Modules                       #
#################################
WriteStatus "Loading PowerCLI Snapin" $Pause
if (!(LoadPowerCLI)) {
	LogErrorExit "Please install the VMWare PowerCLI Tools." 1
}

WriteStatus "Loading DataONTap Module" $Pause
if (!(LoadDataONTap)) {
	LogErrorExit "Please install the DataONTap Powershell Module." 2
}

#################################
# Unmount Volume from  Host     #
#################################
WriteStatus "Umounting $BackupMountPoint" $Pause
if (!(DeleteVolumeMountPoint $BackupMountPoint $true)) {
	LogErrorExit "Error Removing Moint Point:  $BackupMountPoint." 3
}

#################################
# Connect to Virtual Center     #
#################################
WriteStatus "Connecting to Virtual Center:  $VMVCenter" $Pause
if (($VIServer = Connect-VIServer $VMVCenter -ErrorVariable +ErrorMsg -ErrorAction SilentlyContinue) -eq $null) {
	LogError "Failed to Connect to Virtual Center $VMCenter."
	LogErrorExit "+ $ErrorMsg" 4
}

#################################
# Get Current VM                #
#################################
WriteStatus "Getting VM for $env:COMPUTERNAME" $Pause
if (($vm = GetCurrentVM) -eq $null) {
	LogError "Error getting VM for Host:  $($env:COMPUTERNAME)"
	LogErrorExit "+ $ErrorMsg" 5
}

#################################
# Remove VMDK file from VM      #
#################################
WriteStatus "Removing '[$FlexCloneName] $VMDKPath' from $($vm.Name)" $Pause
Get-HardDisk -VM $vm -ErrorVariable +ErrorMsg -ErrorAction SilentlyContinue | Where-Object {$_.Filename -eq "[$FlexCloneName] $VMDKPath"}  | Remove-HardDisk -Confirm:$false -ErrorVariable +ErrorMsg -ErrorAction SilentlyContinue
if ((Get-HardDisk -VM $vm -ErrorVariable +ErrorMsg -ErrorAction SilentlyContinue | Where-Object {$_.Filename -eq "[$FlexCloneName] $VMDKPath"}) -ne $null) {
		LogErrorExit "Error Removing VMDK '[$FlexCloneName] $VMDKPath' from $($vm.Name)" 6	
}

#################################
# Rescan Disks on Host          #
#################################
WriteStatus "Rescanning Disks" $Pause
$Garbage = "ReScan" | DiskPart

#################################
# Get list of All VM Hosts in   #
# Cluster and Remove DataStore  # 
#################################
WriteStatus "Gathering list of VMHosts in the Cluster" $Pause
if (($VMHosts = ($vm | Get-Cluster -ErrorVariable +ErrorMsg -ErrorAction SilentlyContinue | Get-VMHost -State "Connected" -ErrorVariable +ErrorMsg -ErrorAction SilentlyContinue | sort))  -eq $null) {
	LogError "Failed to get hosts in $($vm.Name)'s Cluster."
	LogErrorExit "+ $ErrorMsg" 7
}
WriteStatus "Removing DataStore from $(if ($VMHosts.Count -eq $null) {'1 host'} else {'' + $VMHosts.Count + ' hosts'})" $Pause
foreach ($VMHost in $VMHosts) {
	try {
		Remove-Datastore -VMHost $VMHost -Datastore $FlexCloneName -Confirm:$false -ErrorVariable ErrorMsg -ErrorAction SilentlyContinue
		if ((Get-DataStore -VMHost $VMHost -Name $FlexCloneName -ErrorAction SilentlyContinue) -ne $null) {
				LogError "Error Removing Datastore [$FlexCloneName] from $($VMHost.Name)"
				LogError "+ $ErrorMsg"
		}
	} catch {
		LogError "Error Removing Datastore [$FlexCloneName] from $($VMHost.Name)"
		LogError "+ $_"
	}
}

#################################
# Get NetAppFiler Object        #
#################################
WriteStatus "Connecting to NetApp Filer:  $NAFilerIP" $Pause
if (($NAFiler = Connect-NaController -Name $NAFilerIP -ErrorAction SilentlyContinue -ErrorVariable +ErrorMsg) -eq $null) {
	LogError "Failed to connect to NetApp Controller $NAFilerIP."
	LogErrorExit "+ $ErrorMsg" 8
}

#################################
# Delete FlexClone on NetApp    #
#################################
WriteStatus "Setting FlexClone $FlexCloneName offline" $Pause
if (($FlexCloneVolume = Set-NaVol -Name $FlexCloneName -Controller $NAFiler -Offline -ErrorAction SilentlyContinue -ErrorVariable +ErrorMsg) -eq $null) {
	LogError "Failed to set FlexClone $FlexCloneName to offline state"
	LogErrorExit "+ $ErrorMsg" 9
}

WriteStatus "Removing FlexClone $FlexCloneName from $NAFilerIP" $Pause
Remove-NaVol -Name $FlexCloneVolume -Controller $NAFiler -Confirm:$False -ErrorAction SilentlyContinue -ErrorVariable +ErrorMsg
if ((Get-NaVol -Names $FlexCloneName -ErrorAction SilentlyContinue) -ne $null) {
	LogError "Failed to Remove FlexClone $FlexCloneName from $NAFilerIP"
	LogErrorExit "+ $ErrorMsg" 10
}

####################
# ╔══════════════╗ #
# ║   VICTORY‼   ║ #
# ╚══════════════╝ #
####################
WriteStatus "Flexclone UnMounted and destroyed successfully!" $Pause

